import { ZodType, z } from 'zod';
import { HTTPMethods, FastifyInstance, FastifySchema, FastifyRequest } from 'fastify';
import { FastifyDynamicSwaggerOptions } from '@fastify/swagger';

declare type Models<Key extends string = string> = {
    readonly [K in Key]: ZodType<unknown>;
};
declare type SchemaKey<M extends Models> = M extends Models<infer Key> ? Key & string : never;
declare type SchemaKeyOrDescription<M extends Models> = SchemaKey<M> | {
    readonly description: string;
    readonly key: SchemaKey<M>;
};
declare type SchemaType<M extends Models, Key extends SchemaKey<M>> = z.infer<M[Key]>;
declare type SchemaTypeOption<M extends Models, Key extends void | SchemaKey<M>> = Key extends SchemaKey<M> ? SchemaType<M, Key> : void;

declare type BuildJsonSchemasOptions = {
    readonly $id?: string;
    readonly target?: `jsonSchema7` | `openApi3`;
};
declare type $Ref<M extends Models> = (key: SchemaKeyOrDescription<M>) => {
    readonly $ref: string;
    readonly description?: string;
};
declare type JsonSchema = {
    readonly $id: string;
};
declare type BuildJsonSchemasResult<M extends Models> = {
    readonly schemas: JsonSchema[];
    readonly $ref: $Ref<M>;
};
/**
 * @deprecated
 */
declare const buildJsonSchema: <T>(Type: z.ZodType<T, z.ZodTypeDef, T>, schemaKey: string) => JsonSchema;
declare const buildJsonSchemas: <M extends Models<string>>(models: M, opts?: BuildJsonSchemasOptions) => BuildJsonSchemasResult<M>;

declare type Ref = {
    $ref: string;
};
declare type SpecSchema = {
    readonly properties?: Record<string, unknown>;
    readonly additionalProperties?: boolean | Record<string, unknown>;
    readonly patternProperties?: Record<string, unknown>;
    readonly items?: Record<string, unknown>;
};
declare type SpecSchemas = Record<string, SpecSchema>;
declare type OpenApiSpec = {
    readonly components: {
        readonly schemas: SpecSchemas;
    };
    readonly paths?: Record<string, unknown>;
};
declare type SwaggerSpec = {
    readonly definitions: SpecSchemas;
    readonly paths?: Record<string, unknown>;
};
declare type Spec = OpenApiSpec | SwaggerSpec;
declare type ExtractSchemaPropertiesKey = `properties` | `additionalProperties` | `patternProperties` | `items`;
declare type Timings = {
    readonly begin: number;
    readonly end: number;
    readonly delta: number;
};
declare type TransformWithTimingsResult = {
    readonly spec: Spec;
    readonly timings: {
        readonly rewriteSchemasAbsoluteRefs: Timings;
        readonly extractSchemasProperties: Timings;
        readonly mergeRefs: Timings;
        readonly deleteUnusedSchemas: Timings;
        readonly total: Timings;
    };
};
declare type SchemaKeysOptions = {
    readonly removeInitialSchemasPrefix?: boolean;
    readonly changeCase?: `preserve` | `camelCase` | `PascalCase` | `snake_case` | `param-case`;
};
declare type TransformOptions = {
    readonly rewriteSchemasAbsoluteRefs?: boolean;
    readonly extractSchemasProperties?: boolean | ExtractSchemaPropertiesKey[];
    readonly mergeRefs?: Ref[];
    readonly deleteUnusedSchemas?: boolean;
    readonly schemaKeys?: SchemaKeysOptions;
};
declare class SpecTransformer {
    private readonly DEBUG;
    private readonly spec;
    private readonly initialSchemaKeys;
    private readonly schemasPath;
    constructor(spec: unknown, DEBUG?: boolean);
    private readonly _DEBUG;
    private readonly throw;
    private readonly getSchemaPath;
    private readonly getSchema;
    private readonly getSchemaKeys;
    private readonly createSchemaKey;
    private readonly getAtPath;
    private readonly getAtPathSafe;
    private readonly resolveRef;
    private readonly setAtPath;
    private readonly deleteAtPath;
    private readonly findFirstDeep;
    private readonly hasProperRef;
    private readonly deleteUnusedSchemas;
    private readonly rewriteSchemaAbsoluteRefs;
    private readonly rewriteSchemasAbsoluteRefs;
    private readonly extractSchemaPathAsSchema;
    private readonly extractSchemaPropertiesAtKey;
    private readonly extractSchemaProperties;
    private readonly extractSchemasProperties;
    private readonly mergeRef;
    private readonly mergeRefs;
    readonly transformWithTimings: (opts?: TransformOptions) => TransformWithTimingsResult;
    readonly transform: (opts?: TransformOptions) => Spec;
}

declare type RegisterOptions<S extends Models> = {
    readonly jsonSchemas: BuildJsonSchemasResult<S>;
    readonly swaggerOptions?: FastifyDynamicSwaggerOptions & {
        readonly transformSpec?: {
            readonly cache?: boolean;
            readonly routePrefix?: string;
            readonly options?: TransformOptions;
        };
    };
};
declare type RouteHandlerParams<M extends Models, Params extends void | SchemaKey<M>, Body extends void | SchemaKey<M>, Querystring extends void | SchemaKey<M>> = FastifyRequest<{
    Params: SchemaTypeOption<M, Params>;
    Body: SchemaTypeOption<M, Body>;
    Querystring: SchemaTypeOption<M, Querystring>;
}>;
declare type RouteHandler<M extends Models, Params extends void | SchemaKey<M>, Body extends void | SchemaKey<M>, Reply extends void | SchemaKey<M>, Querystring extends void | SchemaKey<M>> = (params: RouteHandlerParams<M, Params, Body, Querystring>) => Promise<SchemaTypeOption<M, Reply>>;
declare type RouteConfig<M extends Models = Models, Method extends Lowercase<HTTPMethods> = Lowercase<HTTPMethods>, Params extends void | SchemaKey<M> = void, Body extends void | SchemaKey<M> = void, Reply extends void | SchemaKey<M> = void, Querystring extends void | SchemaKey<M> = void> = {
    readonly url: string;
    readonly method: Method;
    readonly operationId: string;
    readonly description?: string;
    readonly params?: Exclude<Params, void> | {
        readonly description: string;
        readonly key: Exclude<Params, void>;
    };
    readonly body?: Exclude<Body, void> | {
        readonly description: string;
        readonly key: Exclude<Body, void>;
    };
    readonly reply?: Exclude<Reply, void> | {
        readonly description: string;
        readonly key: Exclude<Reply, void>;
    };
    readonly querystring?: Exclude<Querystring, void> | {
        readonly description: string;
        readonly key: Exclude<Querystring, void>;
    };
    readonly handler: RouteHandler<M, Params, Body, Reply, Querystring>;
} & FastifySchema;
declare type FastifyZod<M extends Models> = {
    readonly [Method in Lowercase<HTTPMethods>]: <Params extends void | SchemaKey<M> = void, Body extends void | SchemaKey<M> = void, Reply extends void | SchemaKey<M> = void, Querystring extends void | SchemaKey<M> = void>(url: string, config: Omit<RouteConfig<M, Method, Params, Body, Reply, Querystring>, `url` | `method` | `schema` | `handler`>, handler: RouteHandler<M, Params, Body, Reply, Querystring>) => void;
};
declare type FastifyZodInstance<M extends Models> = FastifyInstance & {
    readonly zod: FastifyZod<M>;
};
declare const withRefResolver: (options: FastifyDynamicSwaggerOptions) => FastifyDynamicSwaggerOptions;
declare const register: <S extends Models<string>>(f: FastifyInstance, { jsonSchemas: { schemas, $ref }, swaggerOptions }: RegisterOptions<S>) => FastifyZodInstance<S>;

export { BuildJsonSchemasOptions, BuildJsonSchemasResult, FastifyZod, JsonSchema, RegisterOptions, SpecTransformer, TransformOptions, buildJsonSchema, buildJsonSchemas, register, withRefResolver };
